package cz.upol.db;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

public class EmployeeManager implements AutoCloseable {
	
	private PreparedStatement getAllNames;
	private PreparedStatement getByNameStmt;
	private PreparedStatement insertStmt;
	private PreparedStatement updateStmt;
	private PreparedStatement deleteStmt;
	
	
	public EmployeeManager(Connection con) throws EmployeeDBException {
		try {
			getAllNames = con.prepareStatement("SELECT name FROM employees");
			getByNameStmt = con.prepareStatement("SELECT * FROM employees WHERE name = ?");
			insertStmt = con.prepareStatement("INSERT INTO employees (name, age, salary) VALUES (?, ?, ?)");
			updateStmt = con.prepareStatement("UPDATE employees SET name = ?, age = ?, salary = ? WHERE name = ?");
			deleteStmt = con.prepareStatement("DELETE FROM employees WHERE name = ?");
		} catch (SQLException e) {
			throw new EmployeeDBException("Unable to initialize prepared statements.", e);
		}
	}
	
	public List<String> getEmployeeNames() throws EmployeeDBException {
		List<String> names = new ArrayList<String>();
		try {
			try (ResultSet results = getAllNames.executeQuery()) {
				while (results.next())
					names.add(results.getString(1));
			}
		} catch (Exception e) {
			throw new EmployeeDBException("Unable to list all employees");
		}
		return names;
	}
	

	public Employee getByName(String name) throws EmployeeDBException {
		Employee emp = null;
		try {
			getByNameStmt.setString(1, name);
			try (ResultSet results = getByNameStmt.executeQuery()) {
				if (results.next())
					emp = new Employee(results.getString("name"),
									   results.getInt("age"),
									   results.getInt("salary"));
			}
		} catch (SQLException e) {
			throw new EmployeeDBException("Unable to find an employee", e);
		}
		
		return emp;
	}
	
	public Employee create(String name, int age, int salary) throws EmployeeDBException {
		try {
			insertStmt.setString(1, name);
			insertStmt.setInt(2, age);
			insertStmt.setInt(3, salary);
			insertStmt.executeUpdate();
			 			
			return new Employee(name, age, salary);
		} catch (SQLException e) {
			throw new EmployeeDBException("Unable to create new employee", e);
		}
	}
	
	public void remove(Employee employee) throws EmployeeDBException {
		try {
			deleteStmt.setString(1, employee.getName());
			deleteStmt.executeUpdate();
		} catch (SQLException e) {
			throw new EmployeeDBException("Unable to delete employee", e);
		}
	}
	
	public void commitChanges(Employee employee) throws EmployeeDBException {
		try {
			updateStmt.setString(1, employee.getName());
			updateStmt.setInt(2, employee.getAge());
			updateStmt.setInt(3, employee.getSalary());
			updateStmt.setString(4, employee.getName());
			updateStmt.executeUpdate();
		} catch (SQLException e) {
			throw new EmployeeDBException("Unable to update employee", e);
		}
	}
	
	@Override
	public void close() {
		try {
			getAllNames.close();
			getByNameStmt.close();
			insertStmt.close();
			updateStmt.close();
			deleteStmt.close();
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}
}
